@@ -120,7 +120,7 @@ g_mime_multipart_encrypted_finalize (GObject *object)
 	g_free (mpe->protocol);
 	
 	if (mpe->decrypted)
-		g_mime_object_unref (mpe->decrypted);
+		g_object_unref (mpe->decrypted);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -250,7 +250,7 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	
 	g_mime_object_write_to_stream (content, filtered_stream);
 	g_mime_stream_flush (filtered_stream);
-	g_mime_stream_unref (filtered_stream);
+	g_object_unref (filtered_stream);
 	
 	/* reset the content stream */
 	g_mime_stream_reset (stream);
@@ -258,12 +258,12 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	/* encrypt the content stream */
 	ciphertext = g_mime_stream_mem_new ();
 	if (g_mime_cipher_encrypt (ctx, FALSE, NULL, recipients, stream, ciphertext, err) == -1) {
-		g_mime_stream_unref (ciphertext);
-		g_mime_stream_unref (stream);
+		g_object_unref (ciphertext);
+		g_object_unref (stream);
 		return -1;
 	}
 	
-	g_mime_stream_unref (stream);
+	g_object_unref (stream);
 	g_mime_stream_reset (ciphertext);
 	
 	/* construct the version part */
@@ -274,14 +274,14 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	g_mime_object_set_content_type (GMIME_OBJECT (version_part), content_type);
 	
 	mpe->protocol = g_strdup (ctx->encrypt_protocol);
-	g_mime_object_ref (content);
 	mpe->decrypted = content;
+	g_object_ref (content);
 	
 	/* construct the encrypted mime part */
 	encrypted_part = g_mime_part_new_with_type ("application", "octet-stream");
 	wrapper = g_mime_data_wrapper_new ();
 	g_mime_data_wrapper_set_stream (wrapper, ciphertext);
-	g_mime_stream_unref (ciphertext);
+	g_object_unref (ciphertext);
 	g_mime_part_set_content_object (encrypted_part, wrapper);
 	g_mime_part_set_filename (encrypted_part, "encrypted.asc");
 	g_mime_part_set_encoding (encrypted_part, GMIME_PART_ENCODING_7BIT);
@@ -290,9 +290,9 @@ g_mime_multipart_encrypted_encrypt (GMimeMultipartEncrypted *mpe, GMimeObject *c
 	/* save the version and encrypted parts */
 	/* FIXME: make sure there aren't any other parts?? */
 	g_mime_multipart_add_part (GMIME_MULTIPART (mpe), GMIME_OBJECT (version_part));
-	g_mime_object_unref (GMIME_OBJECT (version_part));
+	g_object_unref (version_part);
 	g_mime_multipart_add_part (GMIME_MULTIPART (mpe), GMIME_OBJECT (encrypted_part));
-	g_mime_object_unref (GMIME_OBJECT (encrypted_part));
+	g_object_unref (encrypted_part);
 	
 	/* set the content-type params for this multipart/encrypted part */
 	g_mime_object_set_content_type_parameter (GMIME_OBJECT (mpe), "protocol", mpe->protocol);
@@ -335,7 +335,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	
 	if (mpe->decrypted) {
 		/* we seem to have already decrypted the part */
-		g_mime_object_ref (mpe->decrypted);
+		g_object_ref (mpe->decrypted);
 		return mpe->decrypted;
 	}
 	
@@ -362,7 +362,7 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 		g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_PROTOCOL_ERROR,
 			     "Failed to decrypt MIME part: protocol error");
 		
-		g_mime_object_unref (version);
+		g_object_unref (version);
 		g_free (content_type);
 		
 		return NULL;
@@ -373,8 +373,8 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	encrypted = g_mime_multipart_get_part (GMIME_MULTIPART (mpe), GMIME_MULTIPART_ENCRYPTED_CONTENT);
 	mime_type = g_mime_object_get_content_type (encrypted);
 	if (!g_mime_content_type_is_type (mime_type, "application", "octet-stream")) {
-		g_mime_object_unref (encrypted);
-		g_mime_object_unref (version);
+		g_object_unref (encrypted);
+		g_object_unref (version);
 		return NULL;
 	}
 	
@@ -393,27 +393,28 @@ g_mime_multipart_encrypted_decrypt (GMimeMultipartEncrypted *mpe, GMimeCipherCon
 	
 	/* get the cleartext */
 	if (g_mime_cipher_decrypt (ctx, ciphertext, filtered_stream, err) == -1) {
-		g_mime_stream_unref (filtered_stream);
-		g_mime_stream_unref (ciphertext);
-		g_mime_stream_unref (stream);
+		g_object_unref (filtered_stream);
+		g_object_unref (ciphertext);
+		g_object_unref (stream);
 		
 		return NULL;
 	}
 	
 	g_mime_stream_flush (filtered_stream);
-	g_mime_stream_unref (filtered_stream);
-	g_mime_stream_unref (ciphertext);
+	g_object_unref (filtered_stream);
+	g_object_unref (ciphertext);
 	
+	g_mime_stream_reset (stream);
 	parser = g_mime_parser_new ();
 	g_mime_parser_init_with_stream (parser, stream);
-	g_mime_stream_unref (stream);
+	g_object_unref (stream);
 	
 	decrypted = g_mime_parser_construct_part (parser);
 	g_object_unref (parser);
 	
 	if (decrypted) {
 		/* cache the decrypted part */
-		g_mime_object_ref (decrypted);
+		g_object_ref (decrypted);
 		mpe->decrypted = decrypted;
 	} else {
 		g_set_error (err, GMIME_ERROR_QUARK, GMIME_ERROR_PARSE_ERROR,
