@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ typedef struct _GMimeMultipartSignedClass GMimeMultipartSignedClass;
 
 enum {
 	GMIME_MULTIPART_SIGNED_CONTENT,
-	GMIME_MULTIPART_SIGNED_SIGNATURE,
+	GMIME_MULTIPART_SIGNED_SIGNATURE
 };
 
 struct _GMimeMultipartSigned {
@@ -68,9 +68,9 @@ int g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *conten
 				  GMimeCipherContext *ctx, const char *userid,
 				  GMimeCipherHash hash, GError **err);
 
-GMimeCipherValidity *g_mime_multipart_signed_verify (GMimeMultipartSigned *mps,
-						     GMimeCipherContext *ctx,
-						     GError **err);
+GMimeSignatureValidity *g_mime_multipart_signed_verify (GMimeMultipartSigned *mps,
+							GMimeCipherContext *ctx,
+							GError **err);
 
 #ifdef __cplusplus
 }
