@@ -24,13 +24,13 @@
 #ifndef __GMIME_STREAM_BUFFER_H__
 #define __GMIME_STREAM_BUFFER_H__
 
+#include <gmime/gmime-stream.h>
+
 #ifdef __cplusplus
 extern "C" {
 #pragma }
 #endif /* __cplusplus */
 
-#include <gmime/gmime-stream.h>
-
 #define GMIME_TYPE_STREAM_BUFFER            (g_mime_stream_buffer_get_type ())
 #define GMIME_STREAM_BUFFER(obj)            (GMIME_CHECK_CAST ((obj), GMIME_TYPE_STREAM_BUFFER, GMimeStreamBuffer))
 #define GMIME_STREAM_BUFFER_CLASS(klass)    (GMIME_CHECK_CLASS_CAST ((klass), GMIME_TYPE_STREAM_BUFFER, GMimeStreamBufferClass))
@@ -56,7 +56,7 @@ struct _GMimeStreamBuffer {
 	unsigned char *buffer;
 	unsigned char *bufptr;
 	unsigned char *bufend;
-	ssize_t buflen;
+	size_t buflen;
 	
 	GMimeStreamBufferMode mode;
 };
