@@ -1,9 +1,9 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Michael Zucchi <notzed@helixcode.com>
- *           Jeffrey Stedfast <fejj@helixcode.com>
+ *  Authors: Michael Zucchi <notzed@ximian.com>
+ *           Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2000 Helix Code, Inc. (www.helixcode.com)
+ *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,8 +30,14 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/param.h> /* for MAXHOSTNAMELEN */
 #include <string.h>
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h> /* for MAXHOSTNAMELEN */
+#else
+#define MAXHOSTNAMELEN 64
+#endif
+#include <sys/types.h>
+#include <unistd.h>
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
@@ -44,14 +50,20 @@
 #include "gmime-charset.h"
 #include "gmime-iconv.h"
 #include "gmime-iconv-utils.h"
-
-#define d(x)
-#define w(x) x
+#include "gmime-host-utils.h"
 
 #ifndef HAVE_ISBLANK
 #define isblank(c) (c == ' ' || c == '\t')
 #endif
 
+#ifdef ENABLE_WARNINGS
+#define w(x) x
+#else
+#define w(x)
+#endif /* ENABLE_WARNINGS */
+
+#define d(x)
+
 #define GMIME_UUENCODE_CHAR(c) ((c) ? (c) + ' ' : '`')
 #define	GMIME_UUDECODE_CHAR(c) (((c) - ' ') & 077)
 
@@ -323,7 +335,7 @@ get_wday (const unsigned char *in, unsigned int inlen)
 		return -1;
 	
 	for (wday = 0; wday < 7; wday++)
-		if (!g_strncasecmp (in, tm_days[wday], 3))
+		if (!strncasecmp (in, tm_days[wday], 3))
 			return wday;
 	
 	return -1;  /* unknown week day */
@@ -355,7 +367,7 @@ get_month (const unsigned char *in, unsigned int inlen)
 		return -1;
 	
 	for (i = 0; i < 12; i++)
-		if (!g_strncasecmp (in, tm_months[i], 3))
+		if (!strncasecmp (in, tm_months[i], 3))
 			return i;
 	
 	return -1;  /* unknown month */
@@ -452,6 +464,32 @@ get_tzone (struct _date_token **token)
 	return -1;
 }
 
+static time_t
+mktime_utc (struct tm *tm)
+{
+	time_t tt;
+	
+	tm->tm_isdst = -1;
+	tt = mktime (tm);
+	
+#if defined (HAVE_TM_GMTOFF)
+	tt += tm->tm_gmtoff;
+#elif defined (HAVE_TIMEZONE)
+	if (tm->tm_isdst > 0) {
+#if defined (HAVE_ALTZONE)
+		tt -= altzone;
+#else /* !defined (HAVE_ALTZONE) */
+		tt -= (timezone - 3600);
+#endif
+	} else
+		tt -= timezone;
+#else
+#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
+#endif
+	
+	return tt;
+}
+
 static time_t
 parse_rfc822_date (struct _date_token *tokens, int *tzone)
 {
@@ -510,14 +548,7 @@ parse_rfc822_date (struct _date_token *tokens, int *tzone)
 		offset = n;
 	}
 	
-	t = mktime (&tm);
-#if defined(HAVE_TIMEZONE)
-	t -= timezone;
-#elif defined(HAVE_TM_GMTOFF)
-	t += tm.tm_gmtoff;
-#else
-#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
-#endif
+	t = mktime_utc (&tm);
 	
 	/* t is now GMT of the time we want, but not offset by the timezone ... */
 	
@@ -547,7 +578,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 	int hour, min, sec, offset, n;
 	struct _date_token *token;
 	struct tm tm;
-	time_t time;
+	time_t t;
 	
 	memset ((void *) &tm, 0, sizeof (struct tm));
 	got_wday = got_month = got_tzone = FALSE;
@@ -630,24 +661,17 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 	
 	d(printf ("\n"));
 		
-	time = mktime (&tm);
-#if defined(HAVE_TIMEZONE)
-	time -= timezone;
-#elif defined(HAVE_TM_GMTOFF)
-	time += tm.tm_gmtoff;
-#else
-#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
-#endif
+	t = mktime_utc (&tm);
 	
 	/* t is now GMT of the time we want, but not offset by the timezone ... */
 	
 	/* this should convert the time to the GMT equiv time */
-	time -= ((offset / 100) * 60 * 60) + (offset % 100) * 60;
+	t -= ((offset / 100) * 60 * 60) + (offset % 100) * 60;
 	
 	if (tzone)
 		*tzone = offset;
 	
-	return time;
+	return t;
 }
 
 #if 0
@@ -753,7 +777,8 @@ g_mime_utils_generate_message_id (const char *fqdn)
 	static unsigned int count = 0;
 	char host[MAXHOSTNAMELEN + 1];
 #ifdef HAVE_GETHOSTBYNAME
-	struct hostent *h = NULL;
+	struct hostent hostbuf;
+	char *buf = NULL;
 #endif
 	char *msgid;
 	
@@ -761,7 +786,18 @@ g_mime_utils_generate_message_id (const char *fqdn)
 #ifdef HAVE_GETHOSTNAME
 		if (gethostname (host, sizeof (host)) == 0) {
 #ifdef HAVE_GETHOSTBYNAME
-			h = gethostbyname (host);
+			size_t buflen = 1024;
+			int retval;
+			
+			buf = g_malloc (buflen);
+			
+			while ((retval = g_gethostbyname_r (host, &hostbuf, buf, buflen, NULL)) == ERANGE && buflen < 8192)
+				buf = g_realloc (buf, (buflen += 1024));
+			
+			if (retval == -1) {
+				g_free (buf);
+				buf = NULL;
+			}
 #endif /* HAVE_GETHOSTBYNAME */
 		} else {
 			host[0] = '\0';
@@ -771,7 +807,8 @@ g_mime_utils_generate_message_id (const char *fqdn)
 #endif /* HAVE_GETHOSTNAME */
 		
 #ifdef HAVE_GETHOSTBYNAME
-		fqdn = h ? h->h_name : (*host ? host : "localhost.localdomain");
+		fqdn = buf != NULL ? hostbuf.h_name : (*host ? host : "localhost.localdomain");
+		g_free (buf);
 #else
 		fqdn = *host ? host : "localhost.localdomain";
 #endif
@@ -797,23 +834,30 @@ decode_msgid (const char **in)
 	char *msgid = NULL;
 	
 	decode_lwsp (&inptr);
-	if (*inptr == '<') {
-		inptr++;
-		decode_lwsp (&inptr);
-		if ((msgid = decode_addrspec (&inptr))) {
-			decode_lwsp (&inptr);
-			if (*inptr != '>') {
-				w(g_warning ("Invalid msg-id; missing '>': %s", *in));
-			} else {
-				inptr++;
-			}
-			
-			*in = inptr;
-		} else {
-			w(g_warning ("Invalid msg-id; missing addr-spec: %s", *in));
-		}
-	} else {
+	if (*inptr != '<') {
 		w(g_warning ("Invalid msg-id; missing '<': %s", *in));
+	} else {
+		inptr++;
+	}
+	
+	decode_lwsp (&inptr);
+	if ((msgid = decode_addrspec (&inptr))) {
+		decode_lwsp (&inptr);
+		if (*inptr != '>') {
+			w(g_warning ("Invalid msg-id; missing '>': %s", *in));
+		} else {
+			inptr++;
+		}
+		
+		*in = inptr;
+	} else {
+		w(g_warning ("Invalid msg-id; missing addr-spec: %s", *in));
+		*in = inptr;
+		while (*inptr && *inptr != '>')
+			inptr++;
+		
+		msgid = g_strndup (*in, inptr - *in);
+		*in = inptr;
 	}
 	
 	return msgid;
@@ -937,16 +981,8 @@ g_mime_references_clear (GMimeReferences **refs)
 }
 
 
-/**
- * g_mime_utils_header_fold:
- * @in: input header string
- *
- * Folds a header according to the rules in rfc822.
- *
- * Returns an allocated string containing the folded header.
- **/
-char *
-g_mime_utils_header_fold (const char *in)
+static char *
+header_fold (const char *in, gboolean structured)
 {
 	gboolean last_was_lwsp = FALSE;
 	register const char *inptr;
@@ -956,20 +992,25 @@ g_mime_utils_header_fold (const char *in)
 	
 	inptr = in;
 	len = strlen (in);
-	if (len <= GMIME_FOLD_LEN)
+	if (len <= GMIME_FOLD_LEN + 1)
 		return g_strdup (in);
 	
 	out = g_string_new ("");
 	outlen = 0;
-	while (*inptr) {
-		len = strcspn (inptr, " \t");
+	while (*inptr && *inptr != '\n') {
+		len = strcspn (inptr, " \t\n");
 		
-		if (outlen + len > GMIME_FOLD_LEN) {			
-			if (last_was_lwsp)
-				g_string_truncate (out, out->len - 1);
-			
-			g_string_append (out, "\n\t");
-			outlen = 1;
+		if (outlen + len > GMIME_FOLD_LEN) {
+			if (outlen > 1) {
+				if (last_was_lwsp) {
+					if (structured)
+						out->str[out->len - 1] = '\t';
+					
+					g_string_insert_c (out, out->len - 1, '\n');
+				} else
+					g_string_append (out, "\n\t");
+				outlen = 1;
+			}
 			
 			/* check for very long words, just cut them up */
 			while (outlen + len > GMIME_FOLD_LEN) {
@@ -987,22 +1028,24 @@ g_mime_utils_header_fold (const char *in)
 			inptr += len;
 			last_was_lwsp = FALSE;
 		} else {
+			last_was_lwsp = TRUE;
 			if (*inptr == '\t') {
 				/* tabs are a good place to fold, odds
-                                   are that this is where the previous
-                                   mailer folded it */
+				   are that this is where the previous
+				   mailer folded it */
 				g_string_append (out, "\n\t");
 				outlen = 1;
 				inptr++;
-				last_was_lwsp = FALSE;
 			} else {
 				g_string_append_c (out, *inptr++);
 				outlen++;
-				last_was_lwsp = TRUE;
 			}
 		}
 	}
 	
+	if (*inptr == '\n' && out->str[out->len - 1] != '\n')
+		g_string_append_c (out, '\n');
+	
 	ret = out->str;
 	g_string_free (out, FALSE);
 	
@@ -1010,6 +1053,51 @@ g_mime_utils_header_fold (const char *in)
 }
 
 
+/**
+ * g_mime_utils_structured_header_fold:
+ * @in: input header string
+ *
+ * Folds a structured header according to the rules in rfc822.
+ *
+ * Returns an allocated string containing the folded header.
+ **/
+char *
+g_mime_utils_structured_header_fold (const char *in)
+{
+	return header_fold (in, TRUE);
+}
+
+
+/**
+ * g_mime_utils_unstructured_header_fold:
+ * @in: input header string
+ *
+ * Folds an unstructured header according to the rules in rfc822.
+ *
+ * Returns an allocated string containing the folded header.
+ **/
+char *
+g_mime_utils_unstructured_header_fold (const char *in)
+{
+	return header_fold (in, FALSE);
+}
+
+
+/**
+ * g_mime_utils_header_fold:
+ * @in: input header string
+ *
+ * Folds a structured header according to the rules in rfc822.
+ *
+ * Returns an allocated string containing the folded header.
+ **/
+char *
+g_mime_utils_header_fold (const char *in)
+{
+	return header_fold (in, TRUE);
+}
+
+
 /**
  * g_mime_utils_header_printf:
  * @format: string format
@@ -1031,7 +1119,7 @@ g_mime_utils_header_printf (const char *format, ...)
 	buf = g_strdup_vprintf (format, ap);
 	va_end (ap);
 	
-	ret = g_mime_utils_header_fold (buf);
+	ret = header_fold (buf, TRUE);
 	g_free (buf);
 	
 	return ret;
@@ -1154,13 +1242,14 @@ g_mime_utils_unquote_string (char *string)
 gboolean
 g_mime_utils_text_is_8bit (const unsigned char *text, size_t len)
 {
-	const unsigned char *c, *inend;
+	register const unsigned char *inptr;
+	const unsigned char *inend;
 	
 	g_return_val_if_fail (text != NULL, FALSE);
 	
 	inend = text + len;
-	for (c = text; c < inend; c++)
-		if (*c > (unsigned char) 127)
+	for (inptr = text; *inptr && inptr < inend; inptr++)
+		if (*inptr > (unsigned char) 127)
 			return TRUE;
 	
 	return FALSE;
@@ -1299,14 +1388,14 @@ rfc2047_decode_word (const unsigned char *in, size_t inlen)
 			*p = '\0';
 		
 		/* slight optimization */
-		if (!g_strcasecmp (charset, "UTF-8"))
+		if (!strcasecmp (charset, "UTF-8"))
 			return g_strndup (decoded, declen);
 		
 		cd = g_mime_iconv_open ("UTF-8", charset);
 		if (cd == (iconv_t) -1) {
 			w(g_warning ("Cannot convert from %s to UTF-8, header display may "
 				     "be corrupt: %s", charset, g_strerror (errno)));
-			charset = g_mime_charset_locale_name ();
+			charset = g_mime_locale_charset ();
 			cd = g_mime_iconv_open ("UTF-8", charset);
 			if (cd == (iconv_t) -1)
 				return NULL;
@@ -1328,15 +1417,134 @@ rfc2047_decode_word (const unsigned char *in, size_t inlen)
 
 
 /**
- * g_mime_utils_8bit_header_decode:
+ * g_mime_utils_header_decode_text:
  * @in: header to decode
  *
- * Decodes and rfc2047 encoded header.
+ * Decodes an rfc2047 encoded 'text' header.
  *
- * Returns the mime encoded header as 8bit text.
+ * Returns the decoded header (which will be in UTF-8 if at all
+ * possible).
  **/
 char *
-g_mime_utils_8bit_header_decode (const unsigned char *in)
+g_mime_utils_header_decode_text (const unsigned char *in)
+{
+	GString *out, *lwsp, *text;
+	const unsigned char *inptr;
+	unsigned char *decoded;
+	gboolean last_was_encoded = FALSE;
+	gboolean last_was_space = FALSE;
+	
+	out = g_string_sized_new (256);
+	lwsp = g_string_sized_new (256);
+	text = g_string_sized_new (256);
+	inptr = in;
+	
+	while (inptr && *inptr) {
+		unsigned char c = *inptr++;
+		
+		if (is_lwsp ((int) c) && !last_was_space) {
+			/* we reached the end of an atom */
+			unsigned char *dword = NULL;
+			const unsigned char *word;
+			gboolean was_encoded;
+			
+			if ((was_encoded = is_rfc2047_encoded_word (text->str, text->len)))
+				word = dword = rfc2047_decode_word (text->str, text->len);
+			else
+				word = text->str;
+			
+			if (word) {
+				if (!(last_was_encoded && was_encoded)) {
+					/* rfc2047 states that you
+                                           must ignore all whitespace
+                                           between encoded words */
+					g_string_append (out, lwsp->str);
+				}
+				
+				g_string_append (out, word);
+				g_free (dword);
+			} else {
+				was_encoded = FALSE;
+				g_string_append (out, lwsp->str);
+				g_string_append (out, text->str);
+			}
+			
+			last_was_encoded = was_encoded;
+			
+			g_string_truncate (lwsp, 0);
+			g_string_truncate (text, 0);
+			
+			if (is_lwsp (c)) {
+				g_string_append_c (lwsp, c);
+				last_was_space = TRUE;
+			} else {
+				/* This is mostly here for interoperability with broken
+                                   mailers that might do something stupid like:
+                                   =?iso-8859-1?Q?blah?=:\t=?iso-8859-1?Q?I_am_broken?= */
+				g_string_append_c (out, c);
+				last_was_encoded = FALSE;
+				last_was_space = FALSE;
+			}
+			
+			continue;
+		}
+		
+		if (!is_lwsp (c)) {
+			g_string_append_c (text, c);
+			last_was_space = FALSE;
+		} else {
+			g_string_append_c (lwsp, c);
+			last_was_space = TRUE;
+		}
+	}
+	
+	if (text->len || lwsp->len) {
+		unsigned char *dword = NULL;
+		const unsigned char *word;
+		gboolean was_encoded;
+		
+		if ((was_encoded = is_rfc2047_encoded_word (text->str, text->len)))
+			word = dword = rfc2047_decode_word (text->str, text->len);
+		else
+			word = text->str;
+		
+		if (word) {
+			if (!(last_was_encoded && was_encoded)) {
+				/* rfc2047 states that you
+				   must ignore all whitespace
+				   between encoded words */
+				g_string_append (out, lwsp->str);
+			}
+			
+			g_string_append (out, word);
+			g_free (dword);
+		} else {
+			g_string_append (out, lwsp->str);
+			g_string_append (out, text->str);
+		}
+	}
+	
+	g_string_free (lwsp, TRUE);
+	g_string_free (text, TRUE);
+	
+	decoded = out->str;
+	g_string_free (out, FALSE);
+	
+	return (char *) decoded;
+}
+
+
+/**
+ * g_mime_utils_header_decode_phrase:
+ * @in: header to decode
+ *
+ * Decodes an rfc2047 encoded 'phrase' header.
+ *
+ * Returns the decoded header (which will be in UTF-8 if at all
+ * possible).
+ **/
+char *
+g_mime_utils_header_decode_phrase (const unsigned char *in)
 {
 	GString *out, *lwsp, *atom;
 	const unsigned char *inptr;
@@ -1484,8 +1692,7 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 	int save = 0;
 	char encoding;
 	
-	
-	if (g_strcasecmp (charset, "UTF-8") != 0)
+	if (strcasecmp (charset, "UTF-8") != 0)
 		cd = g_mime_iconv_open (charset, "UTF-8");
 	
 	if (cd != (iconv_t) -1) {
@@ -1503,7 +1710,7 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 	switch (g_mime_utils_best_encoding (word, len)) {
 	case GMIME_PART_ENCODING_BASE64:
 		enclen = BASE64_ENCODE_LEN (len);
-		encoded = g_alloca (enclen);
+		encoded = g_alloca (enclen + 1);
 		
 		encoding = 'b';
 		
@@ -1518,7 +1725,7 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 		break;
 	case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
 		enclen = QP_ENCODE_LEN (len);
-		encoded = g_alloca (enclen);
+		encoded = g_alloca (enclen + 1);
 		
 		encoding = 'q';
 		
@@ -1538,60 +1745,27 @@ rfc2047_encode_word (GString *string, const unsigned char *word, size_t len,
 }
 
 
-/**
- * g_mime_utils_8bit_header_encode_phrase:
- * @in: header to encode
- *
- * Encodes a header phrase according to the rules in rfc2047.
- *
- * Returns the header phrase as 1 encoded atom. Useful for encoding
- * internet addresses.
- **/
-char *
-g_mime_utils_8bit_header_encode_phrase (const unsigned char *in)
-{
-	const char *charset;
-	GString *string;
-	size_t len;
-	char *str;
-	
-	if (in == NULL)
-		return NULL;
-	
-	len = strlen (in);
-	
-	charset = g_mime_charset_best (in, len);
-	charset = charset ? charset : "iso-8859-1";
-	
-	string = g_string_new ("");
-	
-	rfc2047_encode_word (string, in, strlen (in), charset, IS_ESAFE);
-	
-	str = string->str;
-	g_string_free (string, FALSE);
-	
-	return str;
-}
-
-
-enum _phrase_word_t {
+enum _rfc822_word_t {
 	WORD_ATOM,
+	WORD_QSTRING,
 	WORD_2047
 };
 
-struct _phrase_word {
-	struct _phrase_word *next;
+struct _rfc822_word {
+	struct _rfc822_word *next;
 	const unsigned char *start, *end;
-	enum _phrase_word_t type;
+	enum _rfc822_word_t type;
 	int encoding;
 };
 
 static gboolean
-word_types_compatable (enum _phrase_word_t type1, enum _phrase_word_t type2)
+word_types_compatable (enum _rfc822_word_t type1, enum _rfc822_word_t type2)
 {
 	switch (type1) {
 	case WORD_ATOM:
-		return FALSE;
+		return type2 != WORD_ATOM;
+	case WORD_QSTRING:
+		return type2 != WORD_2047;
 	case WORD_2047:
 		return type2 == WORD_2047;
 	default:
@@ -1599,16 +1773,17 @@ word_types_compatable (enum _phrase_word_t type1, enum _phrase_word_t type2)
 	}
 }
 
-static struct _phrase_word *
-rfc2047_encode_phrase_get_words (const unsigned char *in)
+/* okay, so 'text' fields don't actually contain 'word's, but we can group stuff similarly */
+static struct _rfc822_word *
+rfc2047_encode_get_rfc822_words (const unsigned char *in, gboolean phrase)
 {
 	const unsigned char *inptr, *start, *last;
-	struct _phrase_word *words, *tail, *word;
-	enum _phrase_word_t type = WORD_ATOM;
+	struct _rfc822_word *words, *tail, *word;
+	enum _rfc822_word_t type = WORD_ATOM;
 	int count = 0, encoding = 0;
 	
 	words = NULL;
-	tail = (struct _phrase_word *) &words;
+	tail = (struct _rfc822_word *) &words;
 	
 	last = start = inptr = in;
 	while (inptr && *inptr) {
@@ -1627,7 +1802,7 @@ rfc2047_encode_phrase_get_words (const unsigned char *in)
 		
 		if (g_unichar_isspace (c)) {
 			if (count > 0) {
-				word = g_new (struct _phrase_word, 1);
+				word = g_new (struct _rfc822_word, 1);
 				word->next = NULL;
 				word->start = start;
 				word->end = last;
@@ -1644,9 +1819,13 @@ rfc2047_encode_phrase_get_words (const unsigned char *in)
 			encoding = 0;
 		} else {
 			count++;
-			if (c > 127 && c < 256) {
+			if (phrase && c < 128) {
+				/* phrases can have qstring words */
+				if (!is_atom (c))
+					type = MAX (type, WORD_QSTRING);
+			} else if (c > 127 && c < 256) {
 				type = WORD_2047;
-				encoding = MAX (encoding, 2);
+				encoding = MAX (encoding, 1);
 			} else if (c >= 256) {
 				type = WORD_2047;
 				encoding = 2;
@@ -1657,7 +1836,7 @@ rfc2047_encode_phrase_get_words (const unsigned char *in)
 	}
 	
 	if (count > 0) {
-		word = g_new (struct _phrase_word, 1);
+		word = g_new (struct _rfc822_word, 1);
 		word->next = NULL;
 		word->start = start;
 		word->end = last;
@@ -1671,10 +1850,12 @@ rfc2047_encode_phrase_get_words (const unsigned char *in)
 	return words;
 }
 
+#define MERGED_WORD_LT_FOLDLEN(wlen, type) ((type) == WORD_2047 ? (wlen) < GMIME_FOLD_PREENCODED : (wlen) < (GMIME_FOLD_LEN - 8))
+
 static gboolean
-rfc2047_encode_phrase_merge_words (struct _phrase_word **wordsp)
+rfc2047_encode_merge_rfc822_words (struct _rfc822_word **wordsp)
 {
-	struct _phrase_word *word, *next, *words = *wordsp;
+	struct _rfc822_word *word, *next, *words = *wordsp;
 	gboolean merged = FALSE;
 	
 	/* scan the list, checking for words of similar types that can be merged */
@@ -1685,7 +1866,7 @@ rfc2047_encode_phrase_merge_words (struct _phrase_word **wordsp)
 		while (next) {
 			/* merge nodes of the same type AND we are not creating too long a string */
 			if (word_types_compatable (word->type, next->type)) {
-				if (next->end - word->start < GMIME_FOLD_PREENCODED) {
+				if (MERGED_WORD_LT_FOLDLEN (next->end - word->start, MAX (word->type, next->type))) {
 					/* the resulting word type is the MAX of the 2 types */
 					word->type = MAX (word->type, next->type);
 					
@@ -1716,21 +1897,40 @@ rfc2047_encode_phrase_merge_words (struct _phrase_word **wordsp)
 	return merged;
 }
 
-static char *
-rfc2047_encode_phrase (const unsigned char *in)
+static void
+g_string_append_len_quoted (GString *out, const char *in, size_t len)
 {
-	struct _phrase_word *words, *word, *prev = NULL;
+	register const char *inptr;
+	const char *inend;
+	
+	g_string_append_c (out, '"');
+	
+	inptr = in;
+	inend = in + len;
+	
+	while (inptr < inend) {
+		if ((*inptr == '"') || *inptr == '\\')
+			g_string_append_c (out, '\\');
+		
+		g_string_append_c (out, *inptr);
+		
+		inptr++;
+	}
+	
+	g_string_append_c (out, '"');
+}
+
+static char *
+rfc2047_encode (const unsigned char *in, gushort safemask)
+{
+	struct _rfc822_word *words, *word, *prev = NULL;
 	GString *out;
 	char *outstr;
 	
-	if (in == NULL)
-		return NULL;
+	if (!(words = rfc2047_encode_get_rfc822_words (in, safemask & IS_PSAFE)))
+		return g_strdup (in);
 	
-	words = rfc2047_encode_phrase_get_words (in);
-	if (!words)
-		return NULL;
-	
-	while (rfc2047_encode_phrase_merge_words (&words))
+	while (rfc2047_encode_merge_rfc822_words (&words))
 		;
 	
 	out = g_string_new ("");
@@ -1752,10 +1952,14 @@ rfc2047_encode_phrase (const unsigned char *in)
 		case WORD_ATOM:
 			g_string_append_len (out, word->start, word->end - word->start);
 			break;
+		case WORD_QSTRING:
+			g_assert (safemask & IS_PSAFE);
+			g_string_append_len_quoted (out, word->start, word->end - word->start);
+			break;
 		case WORD_2047:
 			if (prev && prev->type == WORD_2047) {
 				/* include the whitespace chars between these 2 words in the
-                                   resulting rfc2047 encoded word. */
+				   resulting rfc2047 encoded word. */
 				len = word->end - prev->end;
 				start = prev->end;
 				
@@ -1767,10 +1971,9 @@ rfc2047_encode_phrase (const unsigned char *in)
 			}
 			
 			if (word->encoding == 1)
-				rfc2047_encode_word (out, start, len, "iso-8859-1", IS_PSAFE);
+				rfc2047_encode_word (out, start, len, "iso-8859-1", safemask);
 			else
-				rfc2047_encode_word (out, start, len,
-						     g_mime_charset_best (start, len), IS_PSAFE);
+				rfc2047_encode_word (out, start, len, g_mime_charset_best (start, len), safemask);
 			break;
 		}
 		
@@ -1788,19 +1991,98 @@ rfc2047_encode_phrase (const unsigned char *in)
 }
 
 
+/**
+ * g_mime_utils_header_encode_phrase:
+ * @in: header to encode
+ *
+ * Encodes a 'phrase' header according to the rules in rfc2047.
+ *
+ * Returns the encoded 'phrase'. Useful for encoding internet
+ * addresses.
+ **/
+char *
+g_mime_utils_header_encode_phrase (const unsigned char *in)
+{
+	if (in == NULL)
+		return NULL;
+	
+	return rfc2047_encode (in, IS_PSAFE);
+}
+
+
+/**
+ * g_mime_utils_header_encode_text:
+ * @in: header to encode
+ *
+ * Encodes a 'text' header according to the rules in rfc2047.
+ *
+ * Returns the encoded header. Useful for encoding
+ * headers like "Subject".
+ **/
+char *
+g_mime_utils_header_encode_text (const unsigned char *in)
+{
+	if (in == NULL)
+		return NULL;
+	
+	return rfc2047_encode (in, IS_ESAFE);
+}
+
+
+/**
+ * g_mime_utils_8bit_header_decode:
+ * @in: header to decode
+ *
+ * Decodes an rfc2047 encoded header.
+ *
+ * WARNING: This function is deprecated. Use
+ * g_mime_utils_header_decode_text() instead.
+ *
+ * Returns the decoded header (which will be in UTF-8 if at all
+ * possible).
+ **/
+char *
+g_mime_utils_8bit_header_decode (const unsigned char *in)
+{
+	return g_mime_utils_header_decode_text (in);
+}
+
+
 /**
  * g_mime_utils_8bit_header_encode:
  * @in: header to encode
  *
- * Encodes a header according to the rules in rfc2047.
+ * Encodes a 'text' header according to the rules in rfc2047.
  *
- * Returns the header as several encoded atoms. Useful for encoding
+ * WARNING: This function is deprecated. Use
+ * g_mime_utils_header_encode_text() instead.
+ *
+ * Returns the encoded header. Useful for encoding
  * headers like "Subject".
  **/
 char *
 g_mime_utils_8bit_header_encode (const unsigned char *in)
 {
-	return rfc2047_encode_phrase (in);
+	return g_mime_utils_header_encode_text (in);
+}
+
+
+/**
+ * g_mime_utils_8bit_header_encode_phrase:
+ * @in: header to encode
+ *
+ * Encodes a 'phrase' header according to the rules in rfc2047.
+ *
+ * WARNING: This function is deprecated. Use
+ * g_mime_utils_header_encode_phrase() instead.
+ *
+ * Returns the encoded 'phrase'. Useful for encoding internet
+ * addresses.
+ **/
+char *
+g_mime_utils_8bit_header_encode_phrase (const unsigned char *in)
+{
+	return g_mime_utils_header_encode_phrase (in);
 }
 
 
@@ -1813,7 +2095,7 @@ g_mime_utils_8bit_header_encode (const unsigned char *in)
  * @save: leftover bits that have not yet been encoded
  *
  * Base64 encodes the input stream to the output stream. Call this
- * when finished encoding data with #g_mime_utils_base64_encode_step
+ * when finished encoding data with g_mime_utils_base64_encode_step()
  * to flush off the last little bit.
  *
  * Returns the number of bytes encoded.
@@ -2014,7 +2296,7 @@ g_mime_utils_base64_decode_step (const unsigned char *in, size_t inlen, unsigned
  * @save: leftover bits that have not yet been encoded
  *
  * Uuencodes a chunk of data. Call this when finished encoding data
- * with #g_mime_utils_uuencode_step to flush off the last little bit.
+ * with g_mime_utils_uuencode_step() to flush off the last little bit.
  *
  * Returns the number of bytes encoded.
  **/
@@ -2172,7 +2454,7 @@ g_mime_utils_uuencode_step (const unsigned char *in, size_t inlen, unsigned char
  * @save: leftover bits that have not yet been decoded
  *
  * Uudecodes a chunk of data. Performs a 'decode step' on a chunk of
- * uuencoded data. Assumes the "begin <mode> <file name>" line has
+ * uuencoded data. Assumes the "begin mode filename" line has
  * been stripped off.
  *
  * Returns the number of bytes decoded.
@@ -2275,7 +2557,7 @@ g_mime_utils_uudecode_step (const unsigned char *in, size_t inlen, unsigned char
  * @save: leftover bits that have not yet been encoded
  *
  * Quoted-printable encodes a block of text. Call this when finished
- * encoding data with #g_mime_utils_quoted_encode_step to flush off
+ * encoding data with g_mime_utils_quoted_encode_step() to flush off
  * the last little bit.
  *
  * Returns the number of bytes encoded.
